@@ -2,21 +2,24 @@ use std::alloc::Allocator;
 
 use tensorgraph_sys::{
     device::{cpu::Cpu, DefaultDeviceAllocator, Device, DeviceAllocator},
-    ptr::slice::Slice,
+    ptr::reef::Ref,
     vec::Vec,
 };
 
+/// Convert a value into it's owned representation.
 pub trait IntoOwned {
     type Owned;
     fn into_owned(self) -> Self::Owned;
 }
 
-pub trait Storage: AsRef<Slice<Self::T, Self::Device>> {
+/// Represents a storage for [`crate::tensor::Tensor`]
+pub trait Storage: AsRef<Ref<[Self::T], Self::Device>> {
     type T;
     type Device: Device;
 }
 
-pub trait StorageMut: Storage + AsMut<Slice<Self::T, Self::Device>> {}
+/// Represents a mutable storage for [`crate::tensor::Tensor`]
+pub trait StorageMut: Storage + AsMut<Ref<[Self::T], Self::Device>> {}
 
 // Vec
 
@@ -36,12 +39,12 @@ impl<T, A: DeviceAllocator> IntoOwned for Vec<T, A> {
 
 // Shared Slice
 
-impl<'a, T, D: Device> Storage for &'a Slice<T, D> {
+impl<'a, T, D: Device> Storage for &'a Ref<[T], D> {
     type T = T;
     type Device = D;
 }
 
-impl<'a, T: Copy, D: DefaultDeviceAllocator> IntoOwned for &'a Slice<T, D> {
+impl<'a, T: Copy, D: DefaultDeviceAllocator> IntoOwned for &'a Ref<[T], D> {
     type Owned = Vec<T, D::Alloc>;
     fn into_owned(self) -> Self::Owned {
         self.to_owned()
@@ -50,14 +53,14 @@ impl<'a, T: Copy, D: DefaultDeviceAllocator> IntoOwned for &'a Slice<T, D> {
 
 // Mut Slice
 
-impl<'a, T, D: Device> Storage for &'a mut Slice<T, D> {
+impl<'a, T, D: Device> Storage for &'a mut Ref<[T], D> {
     type T = T;
     type Device = D;
 }
 
-impl<'a, T, D: Device> StorageMut for &'a mut Slice<T, D> {}
+impl<'a, T, D: Device> StorageMut for &'a mut Ref<[T], D> {}
 
-impl<'a, T: Copy, D: DefaultDeviceAllocator> IntoOwned for &'a mut Slice<T, D> {
+impl<'a, T: Copy, D: DefaultDeviceAllocator> IntoOwned for &'a mut Ref<[T], D> {
     type Owned = Vec<T, D::Alloc>;
     fn into_owned(self) -> Self::Owned {
         self.to_owned()
